// SoapService.js - in api/services
var soap = require('billon-soap');

var wsdlName = "BsaBase", wsdl = 'data/wsdl/' + wsdlName + '.wsdl', client;

var settings = {
	helloTimeout: 1000,
	soapTimeout: 5000
};

var ValidURL = function(str) {
	var pattern = new RegExp('^(https?:\\/\\/)?'+ // protocol
	'((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|'+ // domain name
	'((\\d{1,3}\\.){3}\\d{1,3}))'+ // OR ip (v4) address
	'(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*'+ // port and path
	'(\\?[;&a-z\\d%_.~+=-]*)?'+ // query string
	'(\\#[-a-z\\d_]*)?$','i'); // fragment locator
	if(!pattern.test(str)) {
		return false;
	} else {
		return true;
	}
};

var errors = {
	ERR_EMPTY_REQUEST_BODY: {
		"err": 'ERR_EMPTY_REQUEST_BODY',
		"description": 'Request body must not be a valid JSON.'
	},
	ERR_MISSING_WUI_DATA: {
		"err": 'ERR_MISSING_WUI_DATA',
		"description": 'Missing WUI parameter in request body.'
	},
	ERR_MISSING_WUI_ADDRESS: {
		"err": 'ERR_MISSING_WUI_ADDRESS',
		"description": 'Missing WUI.Address parameter in request body.'
	},
	ERR_MISSING_WUI_NAME: {
		"err": 'ERR_MISSING_WUI_NAME',
		"description": 'Missing WUI.Name parameter in request body.'
	},
	ERR_INVALID_WUI_DATA_TYPE: {
		"err": 'ERR_INVALID_WUI_DATA_TYPE',
		"description": 'WUI parameter must be an object.'
	},
	ERR_INVALID_WUI_ADDRESS_DATA_TYPE: {
		"err": 'ERR_INVALID_WUI_ADDRESS_DATA_TYPE',
		"description": 'WUI.Address parameter must be a string.'
	},
	ERR_INVALID_WUI_NAME_DATA_TYPE: {
		"err": 'ERR_INVALID_WUI_NAME_DATA_TYPE',
		"description": 'WUI.Name parameter must be a string.'
	},
	ERR_INVALID_WUI_ADDRESS: {
		"err": 'ERR_INVALID_WUI_ADDRESS',
		"description": 'WUI.Address is not a valid URL.'
	},
	ERR_INVALID_PARAMETERS: {
		"err": 'ERR_INVALID_PARAMETERS',
		"description": 'Invalid request body parameters for that request.'
	},
	ERR_WUI_NOT_RESPONDING: {
		"err": 'ERR_WUI_NOT_RESPONDING',
		"description": 'WUI is not responding.'
	},
	ERR_COULD_NOT_HELLO:  {
		"err": 'ERR_COULD_NOT_HELLO',
		"description": 'Hello request failed.'
	},
	ERR_WRONG_WUI_USERNAME:  {
		"err": 'ERR_WRONG_WUI_USERNAME',
		"description": 'Requested WUI has different username then specified.'
	},
	ERR_REQUEST_BODY_IS_NOT_VALID_JSON: {
		"err": 'ERR_REQUEST_BODY_IS_NOT_VALID_JSON',
		"description": 'Request body must be a valid JSON.'
	},
	ERR_COULD_NOT_GET_TASK_STATUS: {
		"err": 'ERR_COULD_NOT_GET_TASK_STATUS',
		"description": 'Could not get task status.'
	},
	ERR_WRONG_WUI_RESPONSE: {
		"err": 'ERR_WRONG_WUI_RESPONSE',
		"description": 'Received wrong response from WUI.'
	}
};

var invalid = function (request) {
	var err = false;
	
	if (!request) {
		err = errors.ERR_EMPTY_REQUEST_BODY;
	} else if (!request.WUI) {
		err = errors.ERR_MISSING_WUI_DATA;
	} else if (!request.WUI.Address) {
		err = errors.ERR_MISSING_WUI_ADDRESS;
	} else if (!request.WUI.Name) {
		err = errors.ERR_MISSING_WUI_NAME;
	} else if (typeof request.WUI != "object") {
		err = errors.ERR_INVALID_WUI_DATA_TYPE;
	} else if (typeof request.WUI.Address != "string") {
		err = errors.ERR_INVALID_WUI_ADDRESS_DATA_TYPE;
	} else if (typeof request.WUI.Name != "string") {
		err = errors.ERR_INVALID_WUI_NAME_DATA_TYPE;
	} else if (!ValidURL(request.WUI.Address)) {
		err = errors.ERR_INVALID_WUI_ADDRESS;
	}
	
	return err;
};

var helloCheck = function(wuiName, cb, options) {
	try {
		client.hello(null, function(err, result) {
			if (err) {
				var error = errors.ERR_COULD_NOT_HELLO;
				error.native_err = err;
				cb(error);
			} else if (!result.return) {
				cb(errors.ERR_WRONG_WUI_RESPONSE);
			} else if (result.return.username != wuiName) {
				cb(errors.ERR_WRONG_WUI_USERNAME);
			} else {
				cb();
			}
		}, options);
	} catch (err) {
		var error = errors.ERR_WUI_NOT_RESPONDING;
		error.native_err = err;
		cb(error);
	}
};

var SoapService = {
	execute: function (method, requestBody, res) {
		this.call[method](requestBody, function(data){
			res.json(data);
		});
	},
	request: function (requestName, requestBody, cb) {
		
		err = invalid(requestBody);
		if (err) {
			cb(err);
			return;
		}
		
		if (!requestBody.params) {
			requestBody.params = null;
		}
		
		client.setEndpoint(requestBody.WUI.Address);
		helloCheck(requestBody.WUI.Name, function(err){
			if (err) {
				cb(err);
				return;
			}
			
			try {
				client[requestName](requestBody.params, function(err, result) {
					if (err) {
						var error = errors.ERR_INVALID_PARAMETERS;
						error.native_err = err;
						cb(error);
					} else {
						//result.return.request = requestName;
						//result.return.WUI = requestBody.WUI;
						var response = result.return;
						if (typeof response.data === "undefined" && typeof requestBody.data !== "undefined") {
							response.data = requestBody.data;
						}
						cb(response);
					}
				}, {timeout: settings.soapTimeout});
			} catch (err) {
				var error = errors.ERR_INVALID_PARAMETERS;
				error.native_err = err;
				cb(error);
			}
		}, {timeout: settings.soapTimeout});		
	},
};

SoapService.call = {
	SoapService: SoapService
};
soap.createClient(wsdl, function(err, cli) {
	if (err) {
		console.log(err);
		return;
	}
	client = cli;
	
	for (var i in client[wsdlName][wsdlName]) {
		SoapService.call[i] = new Function("requestBody", "cb", "this.SoapService.request('"+i+"', requestBody, function(data){cb(data);});");
	}
	
});

module.exports = SoapService;